package br.com.tcc.safeson.safeson.socket;

import com.google.gson.Gson;
import com.navin.flintstones.rxwebsocket.RxWebsocket;

import java.text.SimpleDateFormat;
import java.util.ArrayList;
import java.util.Calendar;
import java.util.HashMap;
import java.util.List;
import java.util.UUID;

import br.com.tcc.safeson.safeson.actions.BaseMessage;
import br.com.tcc.safeson.safeson.models.Trajeto;
import io.reactivex.Scheduler;
import io.reactivex.Single;
import io.reactivex.SingleEmitter;
import io.reactivex.android.schedulers.AndroidSchedulers;
import io.reactivex.schedulers.Schedulers;

public class SocketConnection extends SocketConfig {
    public static SocketConnection main = new SocketConnection();
    public static List<SocketConnection> routes = new ArrayList<>();

    private RxWebsocket websocket;

    // Para direcionar os eventos ouvidos para o correspondente que chamou
    private HashMap<String, SingleEmitter<ResultObject>> singles;
    private HashMap<String, Class> singlesParser;

    private String trajetoUuid;
    private String socialId;
    private Trajeto trajeto;

    public SocketConnection() {
        websocket = new RxWebsocket.Builder()
            .addConverterFactory(WebSocketConverterFactory.create())
            //.addReceiveInterceptor(data -> data)
            .build(getMainConnectionUrl());

        singles = new HashMap<>();
        singlesParser = new HashMap<>();

        listen();
    }

    public SocketConnection(String trajetoUuid, String socialId) {
        websocket = new RxWebsocket.Builder()
            .addConverterFactory(WebSocketConverterFactory.create())
            //.addReceiveInterceptor(data -> data)
            .build(getRouteConnectionUrl(trajetoUuid, socialId));

        this.trajetoUuid = trajetoUuid;
        this.socialId = socialId;

        singles = new HashMap<>();
        singlesParser = new HashMap<>();
        routes.add(this);

        listen();
    }

    public static void dismissAll() {
        routes.clear();
    }

    /**
     * Send a WebSocket message with a Single return value
     * @param data Data to sent
     * @return Single emmiter object
     */
    public Single<ResultObject> sendSingleMessage(BaseMessage data) {
        return Single.create(emitter -> emitSingleWebsocketMessage(emitter, data));
    }

    /**
     * Send a WebSocket message with a Single return value
     * @param data Data to send
     * @param parseResult Objeto de resultado
     * @return Single emmiter object
     */
    public Single<ResultObject> sendSingleMessage(BaseMessage data, Class parseResult) {
        return Single.create(emitter -> emitSingleWebsocketMessage(emitter, data, parseResult));
    }

    /**
     * Send data to websocket and wrap with reactive the response
     * @param emitter SingleEvent emitter
     * @param data Data sent
     */
    private void emitSingleWebsocketMessage(SingleEmitter<ResultObject> emitter, BaseMessage data) {
        singles.put(data.token, emitter);

        websocket
            .send(data)
            .observeOn(Schedulers.newThread())
            .subscribe(
                    System.out::println,
                    this::logError
            );
    }

    /**
     * Send data to websocket and wrap with reactive the response
     * @param emitter SingleEvent emitter
     * @param parser Classe que recebera os dados do result
     * @param data Data sent
     */
    private void emitSingleWebsocketMessage(SingleEmitter<ResultObject> emitter, BaseMessage data, Class parser) {
        singles.put(data.token, emitter);
        singlesParser.put(data.token, parser);

        websocket
            .send(data)
            .observeOn(Schedulers.newThread())
            .subscribe(
                System.out::println,
                    this::logError
            );
    }

    public void connect() {
        if (websocket == null) return;

        websocket
            .connect()
            .observeOn(Schedulers.io())
            .subscribe(
                event -> {
                    System.out.println("Conectado com o WebSocket");
                },
                err -> {
                    System.out.println("Erro na conexão com o websocket -> " + err);
                });
    }

    /**
     * Disconnect from socket
     * CODE LIST: https://tools.ietf.org/html/rfc6455#section-7.4
     */
    public void disconnect() {
        int SUCCESS_CODE = 1000;

        for (int i = 0; i < routes.size(); i++) {
            if (this != routes.get(i)) continue;

            routes.remove(i);
        }

        if (this.trajeto != null) {
            this.trajeto = null;
        }

        websocket.disconnect(SUCCESS_CODE, "Socket desconectado");
    }

    private void emitSingleMessage(ResultObject res) {
        SingleEmitter<ResultObject> emit = singles.get(res.token);
        Class parser = singlesParser.get(res.token);

        if (res.result != null && parser != null) {
            Gson gson = new Gson();
            res.result = gson.fromJson(gson.toJson(res.result), parser);
        }

        if (res.error) {
            emit.onError(new Exception(res.msg));
        } else {
            emit.onSuccess(res);
        }

        singles.remove(res.token);

        if (singlesParser.get(res.token) != null)
            singlesParser.remove(res.token);
    }

    private void listenToMessage(RxWebsocket.Event event) {
        System.out.println("Ouvindo mensagem -> " + event);
        try {
            ResultObject res = ((RxWebsocket.Message) event).data(ResultObject.class);

            // Verifica se a mensagem é do tipo Single
            if (singles.get(res.token) != null) {
                emitSingleMessage(res);
            }

            if (res.broadcast && BroadcastEvent.isListening(res.event)) {
                BroadcastEvent.emitBroadcastMessage(res);
            }
        } catch (Throwable throwable) {
            log("[Mensagem invalida recebida]: " + ((RxWebsocket.Message) event).data().toString());
        }
    }

    private void listen() {
        if (websocket == null) return;

        websocket
            .listen()
            .observeOn(Schedulers.newThread())
            .subscribe(
                event -> {
                    listenToMessage(event);
                },
                err -> {
                    System.out.println("ERRO DE CONEXAO -> " + err);
                }
            );
    }


    private void logError(Throwable throwable) {
        System.out.println(String.format("\n[%s]:[ERROR]%s", getCurrentTime(), throwable.getMessage()));
    }

    private void log(String text) {
        System.out.println(String.format("\n[%s]:%s", getCurrentTime(), text));
    }

    private String getCurrentTime() {
        Calendar c = Calendar.getInstance();
        SimpleDateFormat sdf = new SimpleDateFormat("HH:mm:ss");
        return sdf.format(c.getTime());
    }

    public void setRoute(Trajeto t) {
        this.trajeto = t;
    }

    public String getTrajetoUuid() {
        return this.trajetoUuid;
    }

    public Trajeto getTrajeto() {
        return this.trajeto;
    }

    public static SocketConnection findRouteConnection(String uuid) {
        for (SocketConnection sc : routes) {
            if (sc.trajetoUuid.equals(uuid))
                return sc;
        }

        return null;
    }

    public static SocketConnection myRouteConnection() {
        for (SocketConnection sc : routes) {
            if (sc.trajeto != null)
                return sc;
        }

        return null;
    }
}
